@@ -325,7 +325,7 @@ private:
         size_t cur_len = 0;
 
         wxString longest_sub_string;
-        auto get_longest_sub_string = [longest_sub_string, input](wxString &longest_sub_str, int cur_len, size_t i) {
+        auto get_longest_sub_string = [input](wxString &longest_sub_str, size_t cur_len, size_t i) {
             if (cur_len > longest_sub_str.Len())
                 longest_sub_str = input.SubString(i - cur_len + 1, i);
         };
